import 'dart:async';

import 'package:circumstellar/src/discord_client.dart';
import 'package:circumstellar/src/discord_constants.dart';
import 'package:circumstellar/src/errors/cache_exception.dart';
import 'package:circumstellar/src/models/entity_base.dart';
import 'package:circumstellar/src/models/guild.dart';
import 'package:circumstellar/src/models/guild_text_channel.dart';
import 'package:circumstellar/src/models/message.dart';
import 'package:circumstellar/src/models/user.dart';

/// A helpful utility to automatically manage caches and download models as neccessary
class EntityManager {
  DiscordClient _client;
  Map<String, EntityBase> _cache = {};

  EntityManager(this._client);

  /// Deletes an entity from the cache. Once deleted, if a caller asks for the same entity again it will be redownloaded.
  void deleteEntity(String id) {
    _cache.remove(id);
  }

  /// Updates (or sets) the value for [id] in the cache to [newEntity];
  void updateEntity(String id, EntityBase newEntity, String typeName) {
    _cache[id] = newEntity;
  }

  /// Attempts to retrieve an entity with [id]. If it doesn't exist, it will download the model and save it to cache.
  Future<EntityBase> getEntity(String id, String typeName,
      [Map<String, dynamic> newData]) async {
    EntityBase finalObject = null;

    var cacheResult = _cache[id];
    if (cacheResult != null) {
      finalObject = cacheResult;
      return finalObject;
    }

    if (typeName == 'Guild') {
      finalObject = new Guild(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest.executeRoute(
                    ApiEndpoints.guilds.getGuildWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (typeName == 'GuildTextChannel') {
      finalObject = new GuildTextChannel(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest.executeRoute(
                    ApiEndpoints.channels.getChannelWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (typeName == 'Message') {
      finalObject = new Message(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else
        throw new CacheException('Cannot create Message object from cache');
    }

    if (typeName == 'User') {
      finalObject = new User(_client);
      if (newData != null) {
        await finalObject.updateModel(newData, _client);
      } else {
        await finalObject.updateModel(
            (await _client.rest.executeRoute(
                    ApiEndpoints.users.getUserWithId(id.toString())))
                .json,
            _client);
      }
    }

    if (finalObject != null) this.updateEntity(id, finalObject, typeName);
    return _cache[id];
  }
}
