import 'dart:async';

import 'package:circumstellar/src/discord_client.dart';
import 'package:circumstellar/src/models/guild_channel.dart';
import 'package:circumstellar/src/models/message.dart';
import 'package:circumstellar/src/rest/request_options.dart';

/// Represents a text-supported channel inside of a guild.
class GuildTextChannel extends GuildChannel {
  GuildTextChannel(DiscordClient client) : super(client);

  Future updateModel(Map<String, dynamic> json, DiscordClient client) async {
    await super.updateModel(json, client);
    this.id = json['id'];
    this.type = json['type'] ?? this.type;
    this.topic = json['topic'] ?? this.topic;
    this.sortingPosition = json['position'] ?? this.sortingPosition;
    this.isNsfw = json['nsfw'] ?? this.isNsfw;
    this.lastMessageId = json['last_message_id'] ?? this.lastMessageId;
    this.parentCategoryId = json['parent_id'] ?? this.parentCategoryId;
  }

  // GuildTextChannel properties

  /// The position of this guild in the channel list (sorting position)
  int sortingPosition = 1;

  // TODO: Permission overwrites

  /// The topic of this channel.
  String topic = '';

  /// If this channel is NSFW (Not Safe For Work)
  bool isNsfw = false;

  /// The message ID of the last message sent to this channel.
  String lastMessageId = ''; // TODO: Implement this as a Message entity

  /// The ID of the parent category
  String parentCategoryId = '';

  // TODO: Last Pin Timestamps

  /// Creates a message in this channel.
  Future<Message> createMessage(String content,
          {Map<String, dynamic> embed, bool tts, RequestOptions options}) =>
      this.client.createMessage(this.id, content,
          embed: embed, tts: tts, options: options);
}
